package scalewaysdkgo

// This test file makes sure that all the auto-generated code compiles.

import (
	_ "github.com/scaleway/scaleway-sdk-go/api/account/v2alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/baremetal/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/domain/v2beta1"
	_ "github.com/scaleway/scaleway-sdk-go/api/instance/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/iot/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/k8s/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/lb/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/marketplace/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/rdb/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/registry/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/test/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/vpc/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/vpcgw/v1"
)
